// 01-CodepageToUCS2.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <string>
#include <iostream>
#include <clocale>
#include <cstring>

// this function plays very important role
// in character conversion
void SetDefaultLocale()
{
	// GetSystemDefaultLocaleName function :
	// https://goo.gl/WLLSG3

	// allocate buffer to hold locale name
	std::wstring locale_name(LOCALE_NAME_MAX_LENGTH, L'\0');
	
	int locale_legnth =
		GetSystemDefaultLocaleName(
			&locale_name[0],
			LOCALE_NAME_MAX_LENGTH);

	// if failed to get locale name, then just return
	if (locale_legnth == 0) return;
	
	// https://goo.gl/1A2Hh4
	// set locale of the process or thread
	_wsetlocale(LC_ALL, locale_name.c_str());

	std::wcout << L"locale name: " << locale_name << std::endl;
}

// convert Windows codepage string to UCS2 (or UTF16) encoding
// if fails, returns empty string L""
std::wstring cpstr_to_ucs2(const char* cpstr)
{
	// if cpstr is nullptr, return empty string
	if (cpstr == nullptr) return L"";

	// count of byte of the string to be converted
	int cpstr_len = strlen(cpstr);
	
	if (cpstr_len < 1) return L"";

	// https://goo.gl/upmy99
	// compute the character length of the string
	// when conversion is successful
	int ucs2_len = MultiByteToWideChar(
		CP_ACP, // CP_ACP works only when the process locale
				// is properly set by calling SetDefaultLocale()
		0, cpstr, cpstr_len, NULL, 0);

	// we failed to compute the character length
	// of the string when conversion is successful
	if (ucs2_len == 0) return L"";

	// prepare buffer to hold converted string
	// do not +1 to ucs2_len, because std::wstring
	// manages terminating null internally
	std::wstring ucs2_str(ucs2_len, L'\0');

	int converted = MultiByteToWideChar(
		CP_ACP, // CP_ACP works only when the process locale
				// is properly set by calling SetDefaultLocale()
		0, cpstr, cpstr_len,
		&ucs2_str[0], ucs2_len);
	
	// failed to convert
	if (converted == 0) 
		return L"";
	else
		return ucs2_str;
}

// convert from UCS2 (or UTF16) to Windows copdepage encoding
// if fails, returns empty string ""
std::string ucs2str_to_cpstr(const wchar_t* ucs2str)
{
	if (ucs2str == nullptr) return "";

	// count of character of UCS2 string
	int ucs2_len = wcslen(ucs2str);

	if (ucs2_len < 1) return "";

	// https://goo.gl/acoQBt
	int cpstr_len = WideCharToMultiByte(
		CP_ACP, // this CP_ACP means
				// we want to convert process (or thread)
				// codepage text to UCS2
		0, // dwFlag
		ucs2str,
		ucs2_len, NULL, 0, NULL, NULL);
	
	// we failed to compute the byte count of the string
	// when the conversion is successful
	if (cpstr_len == 0) return "";

	// prepare a string buffer to
	// hold the converted string
	// do not +1 to cpstr_len,
	// because std::string manages terminating null
	std::string cpstr(cpstr_len, '\0');

	int converted = WideCharToMultiByte(
		CP_ACP, 0, ucs2str, ucs2_len, &cpstr[0],
		cpstr_len, NULL, NULL);

	if (converted == 0) 
		return "";
	else
		return  cpstr;
}

int main()
{
	SetDefaultLocale();

	std::string cpstr;

	// get a Windows codepage string
	std::cout << "Input a string: ";
	std::getline(std::cin, cpstr);

	// convert codepage string to UCS2
	std::wstring ucs2str = cpstr_to_ucs2(cpstr.c_str());

	if (ucs2str == L"")
		std::cout << "Failed to convert" << std::endl;
	else
	{

		std::wcout << L"Codepage to UCS2 conversion successful [" << ucs2str << L"]" << std::endl;
		
		std::string reverse_cpstr = ucs2str_to_cpstr(ucs2str.c_str());

		if (reverse_cpstr == "")
			std::cout << "Failed reverse conversion" << std::endl;
		else
		{
			if (cpstr == reverse_cpstr)
				std::cout << "UCS2 to Codepage conversion successful: ["
				<< reverse_cpstr << "]" << std::endl;
			else
				std::cout << "Reverse converted string is different" << std::endl;
		}
	}
	
    return 0;
}

