// 02-UTF8toUCS2.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <string>
#include <iostream>
#include <clocale>
#include <cstring>

// this function plays very important role
// in character conversion
void SetDefaultLocale()
{
	// GetSystemDefaultLocaleName function :
	// https://goo.gl/WLLSG3

	// allocate buffer to hold locale name
	std::wstring locale_name(LOCALE_NAME_MAX_LENGTH, L'\0');

	int locale_legnth =
		GetSystemDefaultLocaleName(
			&locale_name[0],
			LOCALE_NAME_MAX_LENGTH);

	// if failed to get locale name, then just return
	if (locale_legnth == 0) return;

	// https://goo.gl/1A2Hh4
	// set locale of the process or thread
	_wsetlocale(LC_ALL, locale_name.c_str());

	std::wcout << L"locale name: " << locale_name << std::endl;
}

// convert Windows codepage string to UCS2 (or UTF16) encoding
// if fails, returns empty string L""
std::wstring cpstr_to_ucs2(const char* cpstr)
{
	// if cpstr is nullptr, return empty string
	if (cpstr == nullptr) return L"";

	// count of byte of the string to be converted
	int cpstr_len = (int)strlen(cpstr);

	if (cpstr_len < 1) return L"";

	// https://goo.gl/upmy99
	// compute the character length of the string
	// when conversion is successful
	int ucs2_len = MultiByteToWideChar(
		CP_ACP, // CP_ACP works only when the process locale
				// is properly set by calling SetDefaultLocale()
		0, cpstr, cpstr_len, NULL, 0);

	// we failed to compute the character length
	// of the string when conversion is successful
	if (ucs2_len == 0) return L"";

	// prepare buffer to hold converted string
	// do not +1 to ucs2_len, because std::wstring
	// manages terminating null internally
	std::wstring ucs2_str(ucs2_len, L'\0');

	int converted = MultiByteToWideChar(
		CP_ACP, // CP_ACP works only when the process locale
				// is properly set by calling SetDefaultLocale()
		0, cpstr, cpstr_len,
		&ucs2_str[0], ucs2_len);

	// failed to convert
	if (converted == 0)
		return L"";
	else
		return ucs2_str;
}

// convert from UCS2 (or UTF16) to Windows copdepage encoding
// if fails, returns empty string ""
std::string ucs2str_to_cpstr(const wchar_t* ucs2str)
{
	if (ucs2str == nullptr) return "";

	// count of character of UCS2 string
	int ucs2_len = (int)wcslen(ucs2str);

	if (ucs2_len < 1) return "";

	// https://goo.gl/acoQBt
	int cpstr_len = WideCharToMultiByte(
		CP_ACP, // this CP_ACP means
				// we want to convert process (or thread)
				// codepage text to UCS2
		0, // dwFlag
		ucs2str,
		ucs2_len, NULL, 0, NULL, NULL);

	// we failed to compute the byte count of the string
	// when the conversion is successful
	if (cpstr_len == 0) return "";

	// prepare a string buffer to
	// hold the converted string
	// do not +1 to cpstr_len,
	// because std::string manages terminating null
	std::string cpstr(cpstr_len, '\0');

	int converted = WideCharToMultiByte(
		CP_ACP, 0, ucs2str, ucs2_len, &cpstr[0],
		cpstr_len, NULL, NULL);

	if (converted == 0)
		return "";
	else
		return  cpstr;
}

// convert from UCS2 (or UTF16) to UTF8 encoding
// if fails, returns empty string ""
std::string ucs2str_to_utf8str(const wchar_t* ucs2str)
{
	if (ucs2str == nullptr) return "";

	// count of character of UCS2 string
	int ucs2_len = (int)wcslen(ucs2str);

	if (ucs2_len < 1) return "";

	// https://goo.gl/acoQBt
	int cpstr_len = WideCharToMultiByte(
		CP_UTF8, 
		0, // dwFlag
		ucs2str,
		ucs2_len, NULL, 0, NULL, NULL);

	// we failed to compute the byte count of the string
	// when the conversion is successful
	if (cpstr_len == 0) return "";

	// prepare a string buffer to
	// hold the converted string
	// do not +1 to cpstr_len,
	// because std::string manages terminating null
	std::string cpstr(cpstr_len, '\0');

	int converted = WideCharToMultiByte(
		CP_UTF8, 0, ucs2str, ucs2_len, &cpstr[0],
		cpstr_len, NULL, NULL);

	if (converted == 0)
		return "";
	else
		return  cpstr;
}

// convert UTF8 string to UCS2 (or UTF16) encoding
// if fails, returns empty string L""
std::wstring utf8str_to_ucs2(const char* cpstr)
{
	// if cpstr is nullptr, return empty string
	if (cpstr == nullptr) return L"";

	// count of byte of the string to be converted
	int cpstr_len = (int)strlen(cpstr);

	if (cpstr_len < 1) return L"";

	// https://goo.gl/upmy99
	// compute the character length of the string
	// when conversion is successful
	int ucs2_len = MultiByteToWideChar(
		CP_UTF8, 0, cpstr, cpstr_len, NULL, 0);

	// we failed to compute the character length
	// of the string when conversion is successful
	if (ucs2_len == 0) return L"";

	// prepare buffer to hold converted string
	// do not +1 to ucs2_len, because std::wstring
	// manages terminating null internally
	std::wstring ucs2_str(ucs2_len, L'\0');

	int converted = MultiByteToWideChar(
		CP_UTF8, 0, cpstr, cpstr_len,
		&ucs2_str[0], ucs2_len);

	// failed to convert
	if (converted == 0)
		return L"";
	else
		return ucs2_str;
}

// convert Windows codepage to UTF8
std::string cpstr_to_utf8str(const char* cpstr)
{
	/*
		We cannot convert directly from
		codepage to UTF8 .... instead

		We convert 
			codepage to UCS2
			then UCS2 to UTF8
	*/

	// if nullptr, return empty string ""
	if (cpstr == nullptr) return "";

	std::wstring ucs2str = cpstr_to_ucs2(cpstr);
	
	// we failed to convert from codepage encoding to UCS2
	if (ucs2str == L"") return "";

	// converting from UCS2 to UTF8
	return ucs2str_to_utf8str(ucs2str.c_str());
}


// convert UTF8 to codepage encoding
std::string utf8str_to_cpstr(const char* utf8str)
{
	/*
	We cannot convert directly from
	UTF8 to codepage encoding... instead

	We convert
	UTF8 to UCS2
	then UCS2 to codepage encoding
	*/

	if (utf8str == nullptr) return "";

	std::wstring ucs2str = utf8str_to_ucs2(utf8str);

	// if conversion fails, return empty string ""
	if (ucs2str == L"") return "";

	return ucs2str_to_cpstr(ucs2str.c_str());
}

int main()
{
	SetDefaultLocale();

	std::string cpstr;

	// get a Windows codepage string
	std::cout << "Input a string: ";
	std::getline(std::cin, cpstr);

	// convert codepage string to UCS2
	std::wstring ucs2str = cpstr_to_ucs2(cpstr.c_str());

	if (ucs2str == L"")
		std::cout << "Failed to convert" << std::endl;
	else
	{
		std::string utf8str = ucs2str_to_utf8str(ucs2str.c_str());
		if (utf8str != "")
		{
			std::cout << "UCS2 to UTF8 conversion successful" << std::endl;

			std::wstring reverse_ucs2str = utf8str_to_ucs2(utf8str.c_str());

			if (reverse_ucs2str != L"")
			{
				if (ucs2str == reverse_ucs2str)
				{
					std::wcout << L"UCS2 <-> UTF8 conversion success [" <<
						reverse_ucs2str << L"], [" << ucs2str << L"]" << std::endl;
				}
				else
				{
					std::wcout << L"orginal ucs2str and reverse_ucs2str is different"
						<< std::endl;
				}

			}
			else
			{
				std::cout << "UTF8 to UCS2 reverse conversion failed" << std::endl;
			}
		}
		else
		{
			std::cout << "UCS2 to UTF8 conversion failed" << std::endl;
		}

	}

	return 0;
}

